//#include <windows.h>
#include <stdio.h>
#include <string.h>
#include <SDL/SDL.h>
#include "../tinygl/GL/gl.h"
#include "winlnxdefs.h"
#include "Gfx_#1.3.h"
#include "main.h"
#include "texture.h"
#include "../tinygl/GL/nglx.h"

#ifdef USE_GTK
#include <gtk/gtk.h>
#endif

GFX_INFO gfx_Info;

NGLXContext cx;


unsigned long *RDP_PC;
unsigned long inst1, inst2;
int width=320, height=240;
Rdp_status rdp_Status;
SDL_Surface *Ecran;
static int mode_3d = 0;

/******************************************************************
  Function: CaptureScreen
  Purpose:  This function dumps the current frame to a file
  input:    pointer to the directory to save the file to
  output:   none
*******************************************************************/ 
EXPORT void CALL CaptureScreen ( char * Directory )
{
}

/******************************************************************
  Function: ChangeWindow
  Purpose:  to change the window between fullscreen and window 
            mode. If the window was in fullscreen this should 
			change the screen to window mode and vice vesa.
  input:    none
  output:   none
*******************************************************************/ 
EXPORT void CALL ChangeWindow (void)
{
   static int fullscreen = 0;

   fullscreen = 1 - fullscreen;
   Ecran = SDL_SetVideoMode(width, height, 16, 
 0);
   mode_3d = 0;
}

/******************************************************************
  Function: CloseDLL
  Purpose:  This function is called when the emulator is closing
            down allowing the dll to de-initialise.
  input:    none
  output:   none
*******************************************************************/ 
EXPORT void CALL CloseDLL (void)
{
}

/******************************************************************
  Function: DllAbout
  Purpose:  This function is optional function that is provided
            to give further information about the DLL.
  input:    a handle to the window that calls this function
  output:   none
*******************************************************************/ 
EXPORT void CALL DllAbout ( HWND hParent )
{
   char s[] = "Gfx plugin for Mupen64 emulator\nMade by Hacktarux\n";

   printf(s);

}

/******************************************************************
  Function: DllConfig
  Purpose:  This function is optional function that is provided
            to allow the user to configure the dll
  input:    a handle to the window that calls this function
  output:   none
*******************************************************************/ 
EXPORT void CALL DllConfig ( HWND hParent )
{
}

/******************************************************************
  Function: DllTest
  Purpose:  This function is optional function that is provided
            to allow the user to test the dll
  input:    a handle to the window that calls this function
  output:   none
*******************************************************************/ 
EXPORT void CALL DllTest ( HWND hParent )
{
}

/******************************************************************
  Function: DrawScreen
  Purpose:  This function is called when the emulator receives a
            WM_PAINT message. This allows the gfx to fit in when
			it is being used in the desktop.
  input:    none
  output:   none
*******************************************************************/ 
EXPORT void CALL DrawScreen (void)
{
}

/******************************************************************
  Function: GetDllInfo
  Purpose:  This function allows the emulator to gather information
            about the dll by filling in the PluginInfo structure.
  input:    a pointer to a PLUGIN_INFO stucture that needs to be
            filled by the function. (see def above)
  output:   none
*******************************************************************/ 
EXPORT void CALL GetDllInfo ( PLUGIN_INFO * PluginInfo )
{
   PluginInfo->Version = 0x0103;
   PluginInfo->Type = PLUGIN_TYPE_GFX;
   strcpy(PluginInfo->Name, "Mupen64 gfx plugin");
   PluginInfo->NormalMemory = TRUE;
   PluginInfo->MemoryBswaped = TRUE;
}

/******************************************************************
  Function: InitiateGFX
  Purpose:  This function is called when the DLL is started to give
            information from the emulator that the n64 graphics
			uses. This is not called from the emulation thread.
  Input:    Gfx_Info is passed to this function which is defined
            above.
  Output:   TRUE on success
            FALSE on failure to initialise
             
  ** note on interrupts **:
  To generate an interrupt set the appropriate bit in MI_INTR_REG
  and then call the function CheckInterrupts to tell the emulator
  that there is a waiting interrupt.
*******************************************************************/ 
EXPORT BOOL CALL InitiateGFX (GFX_INFO Gfx_Info)
{
   gfx_Info = Gfx_Info;
   printf("DEBUG : pRDRAM %x \n", gfx_Info.RDRAM);

   return TRUE;
}

/******************************************************************
  Function: MoveScreen
  Purpose:  This function is called in response to the emulator
            receiving a WM_MOVE passing the xpos and ypos passed
			from that message.
  input:    xpos - the x-coordinate of the upper-left corner of the
            client area of the window.
			ypos - y-coordinate of the upper-left corner of the
			client area of the window. 
  output:   none
*******************************************************************/ 
EXPORT void CALL MoveScreen (int xpos, int ypos)
{
}

/******************************************************************
  Function: ProcessDList
  Purpose:  This function is called when there is a Dlist to be
            processed. (High level GFX list)
  input:    none
  output:   none
*******************************************************************/ 
EXPORT void CALL ProcessDList(void)
{
   if (!mode_3d)
     {
	mode_3d = 1;
	Ecran = SDL_SetVideoMode(width, height, 16, 0);
     }
   RDP_PC = (unsigned long *)
     (gfx_Info.RDRAM + (*((unsigned long *)(gfx_Info.DMEM + 0xFF0)) & 0x3FFFFF));
   
   do
     {
	inst1 = *RDP_PC;
	dp_command[inst1 >> 24]();
	RDP_PC += 2;
     } while (dp_command[inst1 >> 24] != DP_ENDDL);
	
   //SDL_GL_SwapBuffers();
  
   nglXSwapBuffers();
  
   glClearDepth(1);
   glClearColor(0,0,0,0);
   glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
   
   *gfx_Info.MI_INTR_REG |= 0x20; 
   gfx_Info.CheckInterrupts (); 
}

/******************************************************************
  Function: ProcessRDPList
  Purpose:  This function is called when there is a Dlist to be
            processed. (Low level GFX list)
  input:    none
  output:   none
*******************************************************************/ 
EXPORT void CALL ProcessRDPList(void)
{
}

/******************************************************************
  Function: RomClosed
  Purpose:  This function is called when a rom is closed.
  input:    none
  output:   none
*******************************************************************/ 
EXPORT void CALL RomClosed (void)
{
   free_texture_cache();
   SDL_ShowCursor(0);
   SDL_Quit();
}

/******************************************************************
  Function: RomOpen
  Purpose:  This function is called when a rom is open. (from the 
            emulation thread)
  input:    none
  output:   none
*******************************************************************/ 
EXPORT void CALL RomOpen (void)
{
   SDL_Init(SDL_INIT_VIDEO | SDL_INIT_AUDIO);
   SDL_ShowCursor(0);
   SDL_EnableKeyRepeat(0, 0);
  
   Ecran = SDL_SetVideoMode(320, 240, 16, 0);
   
   width = 320;
   height = 240;
   
    cx = nglXCreateContext(NULL, 0);
    if ( cx == NULL ) printf(" Error Creating GL Context \n");
    else printf("GL Context Created \n");
    nglXMakeCurrent(0, cx);

   init_texture_cache();
}

/******************************************************************
  Function: ShowCFB
  Purpose:  Useally once Dlists are started being displayed, cfb is
            ignored. This function tells the dll to start displaying
			them again.
  input:    none
  output:   none
*******************************************************************/ 
EXPORT void CALL ShowCFB (void)
{
}

/******************************************************************
  Function: UpdateScreen
  Purpose:  This function is called in response to a vsync of the
            screen were the VI bit in MI_INTR_REG has already been
			set
  input:    none
  output:   none
*******************************************************************/ 
#define t16t16(a)    ( 	((a >> 1) & 0x1F) | 	( a & 0xFFC0) 	)
static int mustUpdate = 0;
EXPORT void CALL UpdateScreen (void)
{
   static short *im16, *buf16;
   static int i;
     if (mode_3d != 1 && mustUpdate)
     {
	
	
	im16 = (short *)((char *)gfx_Info.RDRAM + 
			 (*gfx_Info.VI_ORIGIN_REG & 0x3FFFFF));
	buf16 = Ecran->pixels;
	for (i=0; i<(width*height); i+=2)
	  {
	     buf16[i] = t16t16(im16[i+1]);
	     buf16[i+1] = t16t16(im16[i]);
	  }
	SDL_UpdateRect(Ecran, 0, 0, 0, 0);
     
     }
     
  
}

/******************************************************************
  Function: ViStatusChanged
  Purpose:  This function is called to notify the dll that the
            ViStatus registers value has been changed.
  input:    none
  output:   none
*******************************************************************/ 
EXPORT void CALL ViStatusChanged (void)
{
   if ((*gfx_Info.VI_STATUS_REG & 0x3) != 0) mustUpdate = 1;
   else mustUpdate = 0;
}

/******************************************************************
  Function: ViWidthChanged
  Purpose:  This function is called to notify the dll that the
            ViWidth registers value has been changed.
  input:    none
  output:   none
*******************************************************************/ 
EXPORT void CALL ViWidthChanged (void)
{

    width = *gfx_Info.VI_WIDTH_REG;
   height = 3*(*gfx_Info.VI_WIDTH_REG>>2);
   Ecran = SDL_SetVideoMode(width, height, 16, 0);
   mode_3d=0;
}
