#include <stdio.h>
#include <stdlib.h>
//#include <windows.h>
#include "../tinygl/GL/gl.h"
#include "winlnxdefs.h"
#include "Gfx_#1.3.h"
#include "main.h"
#include "texture.h"

int texture_changed[8];

typedef struct _text_cache_struct
{
	unsigned long addr;
	int crc;
} text_cache_struct;

static text_cache_struct *text_cache;
static int text_cache_size_max = 1000;
static int text_cache_size = 1;

void init_texture_cache()
{
	text_cache = (text_cache_struct*)
                     malloc(text_cache_size_max*sizeof(text_cache_struct));
}

void free_texture_cache()
{
	free(text_cache);
}

static unsigned long search_address(int tile)
{
	int i;
	for (i=1; i<=text_cache_size; i++)
		if (text_cache[i].addr == (unsigned long)(rdp_Status.tiles[tile].tmem_addr))
			//if (text_cache[i].crc == rdp_status.timg_crc)
				return i;
	return 0;
}

static void add_address(int tile)
{
	if ((text_cache_size + 1) == text_cache_size_max)
	{
		text_cache_size_max += 1000;
		text_cache = (text_cache_struct*)
	                     realloc(text_cache, text_cache_size_max*sizeof(text_cache_struct));
	}
	text_cache_size++;
	text_cache[text_cache_size].addr = (unsigned long)(rdp_Status.timg_addr+rdp_Status.tiles[tile].tmem);
	text_cache[text_cache_size].crc = rdp_Status.timg_crc;
}

static void decompress_texture(unsigned char **text, int tile, int w, int h)
{
	int i,j;
	unsigned int index;
	unsigned short couleur;
	char stemp[100];

	switch (rdp_Status.tiles[tile].format)
	{
	case 0:
		switch (rdp_Status.tiles[tile].size)
		{
		case 2: // 16 bit
			*text = (unsigned char*)malloc(rdp_Status.intern_w*rdp_Status.intern_h*3);
			for (j=0; j<h; j++) {
				for (i=0; i<w/2; i++) {
					(*text)[j*rdp_Status.intern_w*3+i*6+0]=
						((((unsigned short *)(rdp_Status.timg_addr))[j*w+i*2+1]>>11) & 0x1F)<<3;
					(*text)[j*rdp_Status.intern_w*3+i*6+1]=
						((((unsigned short *)(rdp_Status.timg_addr))[j*w+i*2+1]>>6) & 0x1F)<<3;
					(*text)[j*rdp_Status.intern_w*3+i*6+2]=
						((((unsigned short *)(rdp_Status.timg_addr))[j*w+i*2+1]>>1) & 0x1F)<<3;
		//			(*text)[j*rdp_Status.intern_w*3+i*6+3]=
		//				(((unsigned short *)(rdp_Status.timg_addr))[j*w+i*2+1] & 1) ? 0xff : 0;
					(*text)[j*rdp_Status.intern_w*3+i*6+3]=
						((((unsigned short *)(rdp_Status.timg_addr))[j*w+i*2+0]>>11) & 0x1F)<<3;
					(*text)[j*rdp_Status.intern_w*3+i*6+4]=
						((((unsigned short *)(rdp_Status.timg_addr))[j*w+i*2+0]>>6) & 0x1F)<<3;
					(*text)[j*rdp_Status.intern_w*3+i*6+5]=
						((((unsigned short *)(rdp_Status.timg_addr))[j*w+i*2+0]>>1) & 0x1F)<<3;
					(*text)[j*rdp_Status.intern_w*3+i*6+6]=
						(((unsigned short *)(rdp_Status.timg_addr))[j*w+i*2+0] & 1) ? 0xff : 0;
				}
			}
			break;
		case 3: // 32 bit
			*text = (unsigned char *)malloc(rdp_Status.intern_w*rdp_Status.intern_h*3);
			for (j=0; j<h; j++) {
				for (i=0; i<w;i++) {
					(*text)[j*rdp_Status.intern_w*3+i*3+0]=rdp_Status.timg_addr[j*w*4+i*4+3];
					(*text)[j*rdp_Status.intern_w*3+i*3+1]=rdp_Status.timg_addr[j*w*4+i*4+2];
					(*text)[j*rdp_Status.intern_w*3+i*3+2]=rdp_Status.timg_addr[j*w*4+i*4+1];
					//(*text)[j*rdp_Status.intern_w*3+i*4+3]=rdp_Status.timg_addr[j*w*4+i*4+0];
				}
			}
			break;
		default:
			*text = (unsigned char*)malloc(rdp_Status.intern_w*rdp_Status.intern_h*3);
			printf("texture rgb size:%d", rdp_Status.tiles[tile].size);
		}
		break;
	case 2: // ci
		switch (rdp_Status.tiles[tile].size)
		{
		case 0: // 4 bit
			*text = (unsigned char*)malloc(rdp_Status.intern_w*rdp_Status.intern_h*3);
			for (j=0; j<h; j++) {
				for (i=0; i<w/2; i++) {
					index = (rdp_Status.tiles[tile].tmem_addr[(j*w/2+i)^3] >> 4) & 0xf;
					couleur = ((unsigned short *)(rdp_Status.palette))
						[(rdp_Status.tiles[tile].palette*16 + index)^1];
					(*text)[j*rdp_Status.intern_w*3+i*6+0]=((couleur >> 11) & 0x1F) << 3;
					(*text)[j*rdp_Status.intern_w*3+i*6+1]=((couleur >> 6) & 0x1F) << 3;
					(*text)[j*rdp_Status.intern_w*3+i*6+2]=((couleur >> 1) & 0x1F) << 3;
					//(*text)[j*rdp_Status.intern_w*4+i*8+3]=(couleur & 1) ? 0xff : 0;
					index = rdp_Status.tiles[tile].tmem_addr[(j*w/2+i)^3] & 0xf;
					couleur = ((unsigned short *)(rdp_Status.palette))
						[(rdp_Status.tiles[tile].palette*16 + index)^1];
					(*text)[j*rdp_Status.intern_w*3+i*6+3]=((couleur >> 11) & 0x1F) << 3;
					(*text)[j*rdp_Status.intern_w*3+i*6+4]=((couleur >> 6) & 0x1F) << 3;
					(*text)[j*rdp_Status.intern_w*3+i*6+5]=((couleur >> 1) & 0x1F) << 3;
					//(*text)[j*rdp_Status.intern_w*4+i*8+7]=(couleur & 1) ? 0xff : 0;
				}
			}
			break;
		case 1: // 8 bit
			*text = (unsigned char*)malloc(rdp_Status.intern_w*rdp_Status.intern_h*3);
			for (j=0; j<h; j++) {
				for (i=0; i<w; i++) {
					index = rdp_Status.tiles[tile].tmem_addr[(j*w+i)^3];
					couleur = ((unsigned short *)(rdp_Status.palette))[index^1];
					(*text)[j*rdp_Status.intern_w*3+i*3+0]=((couleur >> 11) & 0x1F) << 3;
					(*text)[j*rdp_Status.intern_w*3+i*3+1]=((couleur >> 6) & 0x1F) << 3;
					(*text)[j*rdp_Status.intern_w*3+i*3+2]=((couleur >> 1) & 0x1F) << 3;
					//(*text)[j*rdp_Status.intern_w*4+i*4+3]=(couleur & 1) ? 0xff : 0;
				}
			}
			break;
		default:
			*text = (unsigned char*)malloc(rdp_Status.intern_w*rdp_Status.intern_h*3);
			printf("texture ci size:%d", rdp_Status.tiles[tile].size);
		}
		break;
	case 3: // ia
		switch (rdp_Status.tiles[tile].size)
		{
		case 1: // 8 bit
			*text = (unsigned char*)malloc(rdp_Status.intern_w*rdp_Status.intern_h*3);
			for (j=0; j<h; j++) {
				for (i=0; i<w; i++) {
					(*text)[j*rdp_Status.intern_w*3+i*3+0]=rdp_Status.timg_addr[(j*w+i)^3]&0xf0;
					(*text)[j*rdp_Status.intern_w*3+i*3+1]=rdp_Status.timg_addr[(j*w+i)^3]&0xf0;
					(*text)[j*rdp_Status.intern_w*3+i*3+2]=rdp_Status.timg_addr[(j*w+i)^3]&0xf0;
					//(*text)[j*rdp_Status.intern_w*4+i*4+3]=(rdp_Status.timg_addr[(j*w+i)^3]&0xf)<<4;
				}
			}
			break;
		default:
			*text = (unsigned char*)malloc(rdp_Status.intern_w*rdp_Status.intern_h*3);
			printf("texture ia size:%d", rdp_Status.tiles[tile].size);
		}
		break;
	default:
		*text = (unsigned char*)malloc(rdp_Status.intern_w*rdp_Status.intern_h*3);
		printf("texture format:%d", rdp_Status.tiles[tile].format);
	}
}

void load_texture(int tile)
{
	int w,h;
	unsigned char *text;
	unsigned long num_cache;
	char stemp[100];

	num_cache = search_address(tile);
	if (num_cache)
		glBindTexture(GL_TEXTURE_2D, num_cache);
	else
	{
		w = (int)(rdp_Status.tiles[tile].lrs - rdp_Status.tiles[tile].uls+1);
		h = (int)(rdp_Status.tiles[tile].lrt - rdp_Status.tiles[tile].ult+1);

		decompress_texture(&text, tile, w, h);
		
		add_address(tile);
		glBindTexture(GL_TEXTURE_2D, text_cache_size);

		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T,GL_REPEAT );
		
		glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_DECAL);
		glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_FASTEST);

		glTexImage2D(GL_TEXTURE_2D, 0, 3, rdp_Status.intern_w, 
rdp_Status.intern_h, 0, GL_RGB, GL_UNSIGNED_BYTE, text);
		free(text);

	   /*sprintf(stemp, "%d,%d", rdp_status.tiles[tile].format, rdp_status.tiles[tile].size);
		printf(NULL, stemp, "coucou", MB_OK);
		SwapBuffers(hDC);

		glClearDepth(1);
		glClearColor(0,0,0,0);
		glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
		glEnable(GL_TEXTURE_2D);
   
		glDisable(GL_ALPHA_TEST);
		glEnable(GL_ALPHA_TEST);
		glAlphaFunc(GL_NOTEQUAL, 0.0);
		glColor3ub(0xff, 0xff, 0xff);
		glDisable(GL_BLEND);

		glMatrixMode(GL_MODELVIEW);
		glPushMatrix();
		glLoadIdentity();
		glMatrixMode(GL_PROJECTION);
		glPushMatrix();
		glLoadIdentity();

		glBegin(GL_QUADS);
		glTexCoord2d(0,0);
		glVertex2f(-0.5f,-0.5f);
		glTexCoord2d(1,0);
		glVertex2f(-0.5f+w/((float)(width/2)),-0.5f);
		glTexCoord2d(1,1);
		glVertex2f(-0.5f+w/((float)(width/2)),-0.5f-h/((float)(height/2)));
		glTexCoord2d(0,1);
		glVertex2f(-0.5f,-0.5f-h/((float)(height/2)));
		glEnd();

		glPopMatrix();
		glMatrixMode(GL_MODELVIEW);
		glPopMatrix();

		MessageBox(NULL, "coucou", "coucou", MB_OK);
		SwapBuffers(hDC);*/
	}
}
