#ifndef __INIAPI_H__
#define __INIAPI_H__


/*
** INI file api functions
** Copyright (C) Nomad Systems 2000
** http://www.nomad.ee
*/

#include <stdio.h>
#define MAXDIR 	256 
#define MAXFILE	256
#define MAXEXT	256
#define MAXPATH		256


/*man
NAME
	INIFIXUP - ini file loader data fixup entry

SYNOPSIS
	typedef struct {
		int type;		// entry type
		char *section,		// ini section name
			*entry;		// ini entry name within section
		void *data;		// pointer to storage
		int slen;		// length of buffer for string types
	} INIFIXUP;

DESCRIPTION
	array of these structures is passed to loadinifile() to load
	and convert the INI file entries. fixup type is one of

		IF_END		// end of list marker
		IF_LONG		// signed long
		IF_ULONG	// unsigned long
		IF_INT		// signed short
		IF_UINT		// unsigned short
		IF_PATH		// pathname (backslash will be added)
		IF_STRING	// string
		IF_BOOL		// 1-byte logical (Yes/No)
		IF_ESTRING	// quoted/escaped string

	slen is only used for string and path types, set it to 0 for
	others

SEE ALSO
	loadinifile()
*/
typedef struct {
	int type;			// entry type
	char *section,		// ini section name
		*entry;			// ini entry name within section
	void *data;			// pointer to storage
	int slen;			// length of buffer for string types
} INIFIXUP;

enum { 	IF_END,			// end of list
		IF_LONG,		// signed long
		IF_ULONG,		// unsigned long
		IF_INT,			// signed short
		IF_UINT,		// unsigned short
		IF_PATH,		// pathname (backslash will be added)
		IF_STRING,		// string
		IF_BOOL,		// 1-byte logical (Yes/No)
		IF_ESTRING		// quoted/escaped string
};

#ifdef __cplusplus
extern "C" {
#endif
extern int loadinifile(char *fn,INIFIXUP *fixups);
extern int getinientry(FILE *fp,char *entry,char *result,long *pos);
extern int getini(char *filename,char *section,char *entry,char *result);
extern int findinisection(FILE *fp,char *section,long *pos);
extern FILE *openinifile(char *fn,char *mode);
extern void closeinifile(FILE *fp);
extern char *iniparse(char *s,char *buf,int buflen);
extern int putini(char *filename,char *section,char *entry,char *text);
extern char *iniparseq(char *s,char *buf,int buflen);

#ifdef __cplusplus
}
#endif

#endif
