/* simple glx like driver for TinyGL and Nano X */
#include "GL/gl.h"
#include "GL/nglx.h"
#include <SDL/SDL.h>
#include "zgl.h"

#define GR_DRAW_ID int
#define GR_GC_ID SDL_Surface *




typedef struct {
    GLContext *gl_context;
    int xsize,ysize;
    GR_DRAW_ID drawable;
    SDL_Surface * gc;
    int pixtype; /* pixel type in TinyGL */
} TinyNGLXContext;

NGLXContext nglXCreateContext(NGLXContext shareList, int flags)
{
  TinyNGLXContext *ctx;

  if (shareList != NULL) {
    gl_fatal_error("No sharing available in TinyGL");
  }
  ctx=gl_malloc(sizeof(TinyNGLXContext));
  if (!ctx)  return NULL;
  ctx->gl_context=NULL;
  return (NGLXContext) ctx;
}

void glXDestroyContext( NGLXContext ctx1 )
{
  TinyNGLXContext *ctx = (TinyNGLXContext *) ctx1;
  if (ctx->gl_context != NULL) {
    glClose();
  }
  gl_free(ctx);
}


/* resize the glx viewport : we try to use the xsize and ysize
   given. We return the effective size which is guaranted to be smaller */

static int glX_resize_viewport(GLContext *c,int *xsize_ptr,int *ysize_ptr)
{
  TinyNGLXContext *ctx;
  int xsize,ysize;
  
  ctx=(TinyNGLXContext *)c->opaque;

  xsize=*xsize_ptr;
  ysize=*ysize_ptr;

  /* we ensure that xsize and ysize are multiples of 2 for the zbuffer. 
     TODO: find a better solution */
  xsize&=~3;
  ysize&=~3;

  if (xsize == 0 || ysize == 0) return -1;

  *xsize_ptr=xsize;
  *ysize_ptr=ysize;

  ctx->xsize=xsize;
  ctx->ysize=ysize;

  /* resize the Z buffer */
  ZB_resize(c->zb,NULL,xsize,ysize);
  return 0;
}

/* we assume here that drawable is a window */
int nglXMakeCurrent( NGLXDrawable drawable, NGLXContext ctx1)
{
  TinyNGLXContext *ctx = (TinyNGLXContext *) ctx1;
  int mode, xsize, ysize;
  ZBuffer *zb;
  SDL_Surface * win_info;
   Uint32 rmask, gmask, bmask, amask;

#if TGL_FEATURE_RENDER_BITS == 32
#if SDL_BYTEORDER == SDL_BIG_ENDIAN
    rmask = 0xff000000;
   gmask = 0x00ff0000;
   bmask = 0x0000ff00;
   amask = 0x000000ff;
#else
    rmask = 0x000000ff;
    gmask = 0x0000ff00;
    bmask = 0x00ff0000;
    amask = 0xff000000;
#endif
#endif

#if TGL_FEATURE_RENDER_BITS == 16
#if SDL_BYTEORDER == SDL_BIG_ENDIAN
    rmask = 0xf8000000;
   gmask = 0x07E0000;
   bmask = 0x001F000;
   amask = 0x00000000;
#else

    rmask =  0x0000001F;
    gmask = 0x000007e0;
    bmask = 0x0000f800;
    amask = 0x00000000;
#endif
#endif



  if (ctx->gl_context == NULL) {
      /* create the TinyGL context */
      //GrGetWindowInfo(drawable, &win_info);
      win_info = SDL_GetVideoSurface();


      xsize = win_info->w;
      ysize = win_info->h;
      
    
      /* currently, we only support 16 bit rendering */
      mode = ZB_MODE_5R6G5B;
      zb=ZB_open(xsize,ysize,mode,0,NULL,NULL,NULL);
      if (zb == NULL) {
          fprintf(stderr, "Error while initializing Z buffer\n");
          exit(1);
      }

      ctx->pixtype = 16 /*MWPF_TRUECOLOR565*/;

      /* create a gc */

    ctx->gc = SDL_CreateRGBSurface(SDL_SWSURFACE, xsize, ysize, 16, rmask, gmask, bmask, amask);
    if(ctx->gc == NULL) {
        fprintf(stderr, "CreateRGBSurface failed: %s\n", SDL_GetError());
        exit(1);
    }
      
      
      /* initialisation of the TinyGL interpreter */
      glInit(zb);
      ctx->gl_context=gl_get_context();
      ctx->gl_context->opaque=(void *) ctx;
      ctx->gl_context->gl_resize_viewport=glX_resize_viewport;

      /* set the viewport : we force a call to glX_resize_viewport */
      ctx->gl_context->viewport.xsize=-1;
      ctx->gl_context->viewport.ysize=-1;
      
      glViewport(0, 0, xsize, ysize);
     }
  
  return 1;
}

void nglXSwapBuffers( void )
{
    GLContext *gl_context;
    TinyNGLXContext *ctx;
      SDL_Surface * win_info;
    /* retrieve the current NGLXContext */
    gl_context=gl_get_context();
    ctx=(TinyNGLXContext *)gl_context->opaque;
	
	//SDL_UnlockSurface(ctx->gc);
	 ZB_copyFrameBuffer(ctx->gl_context->zb,ctx->gc->pixels,ctx->gc->pitch);
	//SDL_LockSurface(ctx->gc);
	
	win_info = SDL_GetVideoSurface();
	//SDL_UpdateRect( ctx->gc , 0,0,0,0);
	
	SDL_UnlockSurface(ctx->gc);
	SDL_BlitSurface( ctx->gc , 0 ,win_info,0);
	SDL_LockSurface(ctx->gc);
	
	SDL_UpdateRect( win_info , 0,0,0,0);
	
	
	    
  //  GrArea(drawable, ctx->gc, 0, 0, ctx->xsize, ctx->ysize, ctx->gl_context->zb->pbuf, ctx->pixtype);
}


